﻿namespace StockSharp.Plaza.Metadata
{
	using System;
	using System.Collections.Generic;
	using System.Linq;

	using Ecng.Collections;
	using Ecng.Common;

	/// <summary>
	/// Коллекция колонок с метаданными.
	/// </summary>
	public sealed class PlazaColumnList : SynchronizedCollection<PlazaColumn>
	{
		private readonly SynchronizedDictionary<PlazaColumn, int> _columnIndecies = new SynchronizedDictionary<PlazaColumn, int>();

		private readonly SynchronizedSet<PlazaColumn> _columns = new SynchronizedSet<PlazaColumn>();
		private readonly SynchronizedSet<PlazaColumn> _nonMandatoryColumns = new SynchronizedSet<PlazaColumn>();

		internal PlazaColumnList()
		{
		}

		private PlazaTable _table;

		internal PlazaTable Table
		{
			get { return _table; }
			set
			{
				if (value == null)
					throw new ArgumentNullException("value");

				foreach (var column in this)
				{
					CheckColumnTable(column, value);
				}

				_table = value;
			}
		}

		internal IEnumerable<PlazaColumn> NonMandatoryColumns
		{
			get { return _nonMandatoryColumns; }
		}

		/// <summary>
		/// Найти колонку по текстовому названию.
		/// </summary>
		/// <param name="name">Название колонки.</param>
		/// <returns>Найденная колонка. Если колонка не найдена, то будет возвращено значение null.</returns>
		public PlazaColumn this[string name]
		{
			get { return this.FirstOrDefault(c => c.Name == name); }
		}

		internal IEnumerable<string> Names
		{
			get { return this.Select(x => x.Name); }
		}

		/// <summary>
		/// Найти индекс для переданной колонки.
		/// </summary>
		/// <param name="column">Колонка.</param>
		/// <returns>Индекс колонки. Если колонка не присуствует в <see cref="PlazaColumnList"/>, то будет возвращено -1.</returns>
		public override int IndexOf(PlazaColumn column)
		{
			return _columnIndecies.TryGetValue2(column) ?? -1;
		}

		/// <summary>
		/// Проверить, добавлена ли колонка.
		/// </summary>
		/// <param name="column">Колонка.</param>
		/// <returns>True, если колонка была ранее добавлена, иначе, false.</returns>
		public override bool Contains(PlazaColumn column)
		{
			return _columns.Contains(column);
		}

		/// <summary>
		/// Вызывается при добавлении колонки в конец списка.
		/// </summary>
		/// <param name="column">Колонка.</param>
		protected override void OnAdding(PlazaColumn column)
		{
			if (column == null)
				throw new ArgumentNullException("column");

			if (Contains(column))
				throw new ArgumentException("Колонка {0} уже добавлена.".Put(column.Name), "column");

			//column = column.Clone();

			_columns.Add(column);

			if (Table != null)
				CheckColumnTable(column, Table);

			if (!column.IsMandatory)
				_nonMandatoryColumns.Add(column);

            _columnIndecies.Add(column, _columnIndecies.Count);
			//column.Index = Count;

			base.OnAdding(column);
		}

		/// <summary>
		/// Вызывается при удалении всех колонок из списка.
		/// </summary>
		protected override void OnClearing()
		{
			throw new NotSupportedException("Нельзя удалять все колонки.");
		}

		/// <summary>
		/// Вызывается при удалении колонки.
		/// </summary>
		/// <param name="column">Колонка.</param>
		protected override void OnRemoving(PlazaColumn column)
		{
			if (column == null)
				throw new ArgumentNullException("column");

			if (column.IsMandatory)
				throw new ArgumentException("Нельзя удалять обязательную колонку {0}.".Put(column.Name));

			_columns.Remove(column);
			_nonMandatoryColumns.Remove(column);

			base.OnRemoving(column);
		}

		/// <summary>
		/// Вызывается при включении колонки в список.
		/// </summary>
		/// <param name="index">Индекс.</param>
		/// <param name="column">Колонка.</param>
		protected override void OnInserting(int index, PlazaColumn column)
		{
			if (Contains(column))
				throw new ArgumentException("Колонка {0} уже добавлена.".Put(column.Name), "column");

			if (Table != null)
				CheckColumnTable(column, Table);

			//column = column.Clone();

			if (!column.IsMandatory)
				_nonMandatoryColumns.Add(column);

			_columns.Add(column);

			base.OnInserting(index, column);
		}

		///// <summary>
		///// Вызывается при добавлении колонки в конец списка.
		///// </summary>
		///// <param name="column">Колонка.</param>
		//protected override void OnAdded(PlazaColumn column)
		//{
		//    Changed.SafeInvoke();
		//    base.OnAdded(column);
		//}

		///// <summary>
		///// Вызывается при удалении колонки.
		///// </summary>
		///// <param name="column">Колонка.</param>
		//protected override void OnRemoved(PlazaColumn column)
		//{
		//    Changed.SafeInvoke();
		//    base.OnRemoved(column);
		//}

		///// <summary>
		///// Вызывается при включении колонки в список.
		///// </summary>
		///// <param name="index">Индекс.</param>
		///// <param name="column">Колонка.</param>
		//protected override void OnInserted(int index, PlazaColumn column)
		//{
		//    Changed.SafeInvoke();
		//    base.OnInserted(index, column);
		//}

		//private void ReIndex()
		//{
		//    var index = 0;

		//    foreach (var column in this)
		//        column.Index = index++;
		//}

		private static void CheckColumnTable(PlazaColumn column, PlazaTable table)
		{
			if (column == null)
				throw new ArgumentNullException("column");

            if (column.TableId != table.Id)
				throw new ArgumentException("Колонка {0} относиться к таблице {1}, в то время как сама таблица имеет системное имя {2}.".Put(column.Name, column.TableId, table.Id));
		}
	}
}