﻿namespace StockSharp.Plaza
{
	using System;
	using System.Runtime.InteropServices;

	using Ecng.Common;

	using StockSharp.Plaza.Metadata;
	using StockSharp.Plaza.Wrappers;

	/// <summary>
	/// Состояния потока данных.
	/// </summary>
	public enum PlazaStreamStates
	{
		/// <summary>
		/// Закрыт.
		/// </summary>
		Closed,

		/// <summary>
		/// Открыт.
		/// </summary>
		Opened,
	}

    /// <summary>
	/// Группы инструментов.
	/// </summary>
    public enum SecurityGrops
	{
		/// <summary>
        /// Фьючерсы.
		/// </summary>
        Future,

		/// <summary>
		/// Опционы.
		/// </summary>
        Option,

        /// <summary>
        /// Общее.
        /// </summary>
        Common 
	}

	/// <summary>
	/// Поток данных сервера Плазы.
	/// </summary>
	public class PlazaStream : Disposable
	{
        internal PlazaStream(PlazaTable table, SecurityGrops securityType = SecurityGrops.Common)
		{
			if (table == null)
				throw new ArgumentNullException("table");

			Table = table;

        	NativeStream = WrapperHelper.CreateStream();
			NativeStream.ConnectionString = string.Empty;
        	NativeStream.Type = StreamWrapperRequests.CombinedDynamic;
        	NativeStream.StreamName = Table.Id.StreamName;

			NativeStream.StreamDataBegin += OnStreamDataBegin;
			NativeStream.StreamDataEnd += OnStreamDataEnd;
			NativeStream.StreamDatumDeleted += OnStreamDatumDeleted;
			NativeStream.StreamStateChanged += OnStreamStateChanged;
			NativeStream.StreamDataInserted += OnStreamDataInserted;
			NativeStream.StreamDataDeleted += OnStreamDataDeleted;
			NativeStream.StreamLifeNumChanged += OnStreamLifeNumChanged;

            SecurityGroup = securityType;
		}

		/// <summary>
		/// Способ получения данных для потока репликации.
		/// </summary>
		public StreamWrapperRequests RequestType
		{
			get { return NativeStream.Type; }
			set { NativeStream.Type = value; }
		}

		/// <summary>
		/// Таблица с метаданными, описывающая данные этого потока.
		/// </summary>
		public PlazaTable Table { get; private set; }

		/// <summary>
		/// Название конфигурационного файла, где будут сохранены настройки по потоку.
		/// </summary>
		public string ConfigFileName { get; set; }

		private int _revision;

		/// <summary>
		/// Служебный номер, хранящий в себе количество изменений потока Plaza.
		/// </summary>
		/// <remarks>Значение по умолчанию равно 0.</remarks>
		public int Revision
		{
			get { return _revision; }
			set
			{
				if (_revision != value)
				{
					_revision = value;
					RevisionChanged.SafeInvoke();
				}
			}
		}

		private IStreamWrapper _nativeStream;

		private IStreamWrapper NativeStream
		{
			get
			{
				if (_nativeStream == null)
					throw new InvalidOperationException("Поток '{0}' не инициализирован.".Put(Table.Name));

				return _nativeStream;
			}
			set { _nativeStream = value; }
		}

		private PlazaStreamStates _state;

		/// <summary>
		/// Состояние потока.
		/// </summary>
		public PlazaStreamStates State
		{
			get { return _state; }
			private set
			{
				if (value != _state)
				{
					_state = value;
					StateChanged.SafeInvoke();
				}
			}
		}

        /// <summary>
        /// Группы инструментов для которых предназначен поток данных
        /// </summary>
        public SecurityGrops SecurityGroup { get; private set; }

		/// <summary>
		/// Проверить, находится ли поток в состоянии получении онлайн-данных.
		/// </summary>
		/// <returns>
		/// Находится ли поток в состоянии получении онлайн-данных.
		/// </returns>
		public bool IsOnline()
		{
			return NativeStream != null && NativeStream.State == StreamWrapperStates.Online;
		}

		/// <summary>
		/// Событие изменения <see cref="State"/>.
		/// </summary>
		public event Action StateChanged;

		/// <summary>
		/// Событие сброса состояния всего потока.
		/// </summary>
		public event Action<PlazaStream> Flushed;

		/// <summary>
		/// Событие смены номера ревизии <see cref="Revision"/>.
		/// </summary>
		public event Action RevisionChanged;

		/// <summary>
		/// Событие переподключения.
		/// </summary>
		public event Action ReConnected;

		/// <summary>
		/// Событие начала обработки данных.
		/// </summary>
		public event Action Begin;

		/// <summary>
		/// Событие окончания обработки данных.
		/// </summary>
		public event Action<PlazaStream> End;

		/// <summary>
		/// Событие удаления данных.
		/// </summary>
		public event Action<PlazaStream, long> Cleared;

		/// <summary>
		/// Событие вставки новых данных.
		/// </summary>
		public event Action<PlazaRecord> Inserted;

		/// <summary>
		/// Событие удаления данных.
		/// </summary>
		public event Action<long, PlazaRecord> Deleted;

		/// <summary>
		/// Событие о записи в лог.
		/// </summary>
		public event Action<PlazaStream, string> Log;

		private void SetRevision()
		{
			NativeStream.TableSet.UpdateRevision(Table.Id.TableName, Revision);
			Log.SafeInvoke(this, "Ревизия установлена в {0}.".Put(Revision));
		}

		private void OnStreamDatumDeleted(IStreamWrapper stream, string tableName, long revision)
		{
			// P2ClientGate.doc - стр. 18:
			// Специальное уведомление StreamDatumDeleted присылается сервером по каждой таблице в начале синхронизации. 
			// Это уведомление означает «данных с ревиженами меньше указанного, на сервере нет».
			// «Безбазовый» клиент должен удалить все данные с ревиженами, меньшими указанного в уведомлении из своего хранилища.
			RaiseDatumDeleted(revision);
		}

		/// <summary>
		/// Вызвать событие <see cref="Cleared"/>.
		/// </summary>
		/// <param name="revision">Номер ревизии.</param>
		protected virtual void RaiseDatumDeleted(long revision)
		{
			Cleared.SafeInvoke(this, revision);
		}

		private void OnStreamDataBegin(IStreamWrapper stream)
		{
			RaiseDataBegin();
		}

		/// <summary>
		/// Вызвать событие <see cref="Begin"/>.
		/// </summary>
		protected virtual void RaiseDataBegin()
		{
			Begin.SafeInvoke();
		}

		private void OnStreamDataEnd(IStreamWrapper stream)
		{
			RaiseDataEnd();
		}

		/// <summary>
		/// Вызвать событие <see cref="End"/>.
		/// </summary>
		protected virtual void RaiseDataEnd()
		{
			End.SafeInvoke(this);
		}

		private void OnStreamStateChanged(IStreamWrapper stream, StreamWrapperStates newState)
		{
			System.Diagnostics.Trace.WriteLine("{0} - StreamStateChanged: {1}".Put(Table.Id, newState));
			Log.SafeInvoke(this, "StreamStateChanged: State {0}".Put(newState));
			//StateChanged.SafeInvoke();
		}

		private void OnStreamLifeNumChanged(IStreamWrapper stream, int lifeNum)
		{
			System.Diagnostics.Trace.WriteLine("StreamLifeNumChanged: CURRENT LifeNum {0}, NEW LifeNum {1}".Put(NativeStream.TableSet.LifeNum, lifeNum));
			Log.SafeInvoke(this, "StreamLifeNumChanged: LifeNum {0}".Put(lifeNum));

			Revision = 0;

			// P2ClientGate.doc стр. 18:
			// При получении этого уведомления «базовый» клиент очищает базу, изменяет номер жизни во внутренних структурах так, 
			// чтобы он соответствовал серверному и переоткрывает поток (переходит в состояние ReOpen). 
			// ВАЖНО! При получении этого уведомления клиентский код должен вызвать P2TableSet-> SetLifeNumToIni 
			// для сохранения номера жизни в ini-файле схемы!
			NativeStream.TableSet.LifeNum = lifeNum;
			NativeStream.TableSet.UpdateLifeNumber(ConfigFileName);

			RaiseFlushed();
		}

		/// <summary>
		/// Вызвать событие <see cref="Flushed"/>.
		/// </summary>
		protected virtual void RaiseFlushed()
		{
			Flushed.SafeInvoke(this);
		}

		private void OnStreamDataInserted(IStreamWrapper stream, string tableName, IRecordWrapper record)
		{
			var replRev = record.GetInt("replRev");

			if (Revision < replRev)
				Revision = replRev;

			RaiseDataInserted(new PlazaRecord(Table).Fill(record));
		}

		/// <summary>
		/// Вызвать событие <see cref="Inserted"/>.
		/// </summary>
		/// <param name="record">Запись.</param>
		protected virtual void RaiseDataInserted(PlazaRecord record)
		{
			Inserted.SafeInvoke(record);
		}

		private void OnStreamDataDeleted(IStreamWrapper stream, string tableName, long id, IRecordWrapper record)
		{
			RaiseDataDeleted(id, new PlazaRecord(Table).Fill(record));
		}

		/// <summary>
		/// Вызвать событие <see cref="Deleted"/>.
		/// </summary>
		/// <param name="id">Номер записи.</param>
		/// <param name="record">Запись.</param>
		protected virtual void RaiseDataDeleted(long id, PlazaRecord record)
		{
			Deleted.SafeInvoke(id, record);
		}

		internal void ProcessConnection(IConnectionWrapper connection)
		{
			switch (State)
			{
				case PlazaStreamStates.Closed:
					if (ConfigFileName.IsEmpty())
						throw new InvalidOperationException("ConfigFileName is null or empty");

					NativeStream.TableSet = WrapperHelper.CreateTableSet();
					NativeStream.TableSet.Init(ConfigFileName, Table.ReplicationScheme);

					SetRevision();

					State = PlazaStreamStates.Opened;

					Log.SafeInvoke(this, "Поток открыт.");
					break;
				case PlazaStreamStates.Opened:
					if (NativeStream.State == StreamWrapperStates.Error || NativeStream.State == StreamWrapperStates.Closed)
					{
						Log.SafeInvoke(this, "Состояние потока {0}.".Put(NativeStream.State));

						if (NativeStream.State == StreamWrapperStates.Error)
							NativeStream.Close();

						SetRevision();

						NativeStream.Open(connection);
						ReConnected.SafeInvoke();

						Log.SafeInvoke(this, "Поток переоткрыт.");
					}
					break;
				default:
					throw new ArgumentOutOfRangeException();
			}
		}

		internal void Close()
		{
			if (NativeStream == null || NativeStream.State == StreamWrapperStates.Closed)
				return;

			try
			{
				NativeStream.Close();
				State = PlazaStreamStates.Closed;
			}
			catch (COMException /*e*/)
			{
				// Ничего не делаем
				//System.Diagnostics.Trace.WriteLine("Exception {0:X}: {1}".Put(e.ErrorCode, e.Message));
			}
		}

		/// <summary>
		/// Освободить занятые ресурсы.
		/// </summary>
		protected override void DisposeManaged()
		{
			if (NativeStream == null || NativeStream.State == StreamWrapperStates.Closed)
				return;

			try
			{
				NativeStream.StreamDataBegin -= OnStreamDataBegin;
				NativeStream.StreamDataEnd -= OnStreamDataEnd;
				NativeStream.StreamDatumDeleted -= OnStreamDatumDeleted;
				NativeStream.StreamStateChanged -= OnStreamStateChanged;
				NativeStream.StreamDataInserted -= OnStreamDataInserted;
				NativeStream.StreamDataDeleted -= OnStreamDataDeleted;
				NativeStream.StreamLifeNumChanged -= OnStreamLifeNumChanged;

				NativeStream.Close();

				NativeStream = null;
			}
			catch (COMException /*e*/)
			{
				// Ничего не делаем
				//System.Diagnostics.Trace.WriteLine("Exception {0:X}: {1}".Put(e.ErrorCode, e.Message));
			}
		}

		/// <summary>
		/// Получить строковое представление.
		/// </summary>
		/// <returns>Строковое представление.</returns>
		public override string ToString()
		{
			return Table.ToString();
		}
	}
}